#To use the code, change the filenames where file names are called for. The input files should be included in the supplmentary material.

# Begin initialize-------------------------------------------------------------------

#check for presence of, install, and load needed packages. You must have a repository specified to install.
check.packages <- function(pkg){
  new.pkg <- pkg[!(pkg %in% installed.packages()[, "Package"])]
  if (length(new.pkg)) 
    install.packages(new.pkg, dependencies = TRUE)
  sapply(pkg, require, character.only = TRUE)
}
listpacks = c("raster", "spatstat","SDMTools", "sp", "maptools", "TSP", "data.table","tidyr")
check.packages(listpacks)

#input files
in_text<- read.csv("T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\Schuykill_results_UTM_coords_for_R.csv")

#look at the data exported from the Access form
head(in_text)
in_text[1,2]
summary(in_text)
length(in_text[,1]) #2099 rows = 2099 plots

in_sp<- readShapeSpatial("T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\pts_final_schuykill_utmcoords_point_utm.shp")

#look at the geographic data
summary(in_sp)
length(in_sp@data[,1])
#109148 rows; 109148/52 = 2099. So there are 2099 plots, 52 rows per plot. 

#need to prepare for join by editing one of the key columns:
in_sp@data$ctrl_boxb =paste("Box",as.character(in_sp@data$ctrl_box),sep="")

#start of loop that calculates results tables for the full dataset, and the reduced dataset: 
for (s in c(0,1)){
#Set this to "0" if I am using the full, original dataset. Rerun, with this set to 1 for the reduced set (the degradation simulation)
df1red_is_on = s #turn off if not doing data reduction

# format the access output into a normal df -------------------------------
#go through each ctrl_list entry in in_text, make it a vector, and append it; fill in corresponding FID. The point is to make a file to join to the shapefile of each point and thus label each of the plots interpreted (subpoints) with a tree yes/no flag.
#The conditional statement below will simply exclude plots that are 100% non-treed. 
#make empty DF first
df1=data.frame(fid = integer(),ctrl = as.character(), comments = as.character(), count = as.integer(),stringsAsFactors = FALSE)
#run the loop that formats the data correctly -- one row per forested point.
for (i in 1:length(in_text[,1])){
  t=as.vector(strsplit(as.character(in_text[i,2]), ",")[[1]])
  if(length(t)==0){}
  else{
    df1=rbind(df1,data.frame("ctrl"=as.character(t),"fid"=in_text[i,8],"comments"=in_text[i,3],count=in_text[i,1],stringsAsFactors = FALSE),stringsAsFactors = FALSE)
    }
}

names(df1)
length(df1[,1]) #67375 #this is one row per point that has a tree on it
length(unique(df1$fid)) #1602 plots that have at least 1 point with F

# #Implement the degradation simulation -- need to initialize df1red_is_on above ------------------------------
if(df1red_is_on == 1){
  set.seed(12345)
  df1= cbind(df1,rnd =runif(dim(df1)[1],min=0,max=1)) 
  df1$use[df1$rnd >= 0.99] = 0
  df1$use[df1$rnd < 0.99] = 1 #I just made an indicator to remove 1% of points with trees on them.
  df1= df1[df1$use == 1,] 
  }

summary(df1)

#need to prepare for join by editing one of the key columns:
in_sp@data$ctrl_boxb =paste("Box",as.character(in_sp@data$ctrl_box),sep="")


# Join the points with tree cover to the set of all points ----------------
#now do a join: only joining points from the shapefile (in_sp) to the data frame consisting of one row per point touching forest cover. This will create a subset of points (exported below) that I can turn into a shapefile and run the near table command on in arcmap if I want to compare it with R results.


nearnn=merge(df1,in_sp@data, by.x=c("ctrl","fid"),by.y=c("ctrl_boxb","FID_"), all.x=TRUE)
summary(nearnn)#there should only be NAs if it's the reduced subset (see above)
nearnn = nearnn[!is.na(nearnn$ptXutm),]

length(nearnn[,1]) #67375 -- here I've merged the spatial and nonspatial info so I have one row per point that has forest on it. There are no rows where it's 100% nf

totadj = 170
#(totadj=(sum(pairdist(nearnn$ptXutm[nearnn$fid == 0], nearnn$ptYutm[nearnn$fid == 0])<18)-52)/2) #will get  the number of possible f-f adjacencies on the plot (170); this is unidirectional, i.e., 2-1 isn't counted with 1-2. Note I also subtract away the 52 zeros in the diagonal. 
#note that there is a slight discrepancy, for example, between fid 1 and fid 0 -- the distances are slightly off. It has something to do with rounding error, but everything works out.

#first make the container data frame
df=data.frame(fid = integer(),dist = numeric(), cntpts = integer(),tourlength = numeric(), totadjacencies = integer(), observedadjacencies = integer(), nndistance = numeric(),stringsAsFactors = FALSE)
for (i in unique(nearnn$fid)) {
  set.seed(12345)
  #here's the code for the TSP index -- the steps to create the shortest possible route (via the farthest insertion algorith) between all points, returning from the last point back to the first point by shortest route.
  dm = crossdist(nearnn$ptXutm[nearnn$fid == i],
                 nearnn$ptYutm[nearnn$fid == i],
                 nearnn$ptXutm[nearnn$fid == i],
                 nearnn$ptYutm[nearnn$fid == i])
  dmtsp = as.TSP(dm)
  dmtsprout = solve_TSP(dmtsp, method = "farthest_insertion")
  tl = tour_length(dmtsprout)
  
  #here I calculate the nearest neighbor distance:
  nndistance = min(dm[dm > 0])
  
  # here I get the distances for each point to every other point for each plot, and then calculate average values per plot. 
  lfid = length(nearnn$ptXutm[nearnn$fid == i]) #number of points associated with an fid (plot identifier)
  #To get the average interpoint distance
  dist = sum(pairdist(nearnn$ptXutm[nearnn$fid == i], nearnn$ptYutm[nearnn$fid == i])) /
    (lfid ^ 2 - lfid) #note I dealt with the bidirectional nature of the matrix by using lfid^2, and I got correct n by subtracting the count of the diagonal (0) entries, lfid)
  
  #here I get number of observed adjacencies:
  obsadj = ((sum(pairdist(nearnn$ptXutm[nearnn$fid == i], nearnn$ptYutm[nearnn$fid == i]) < 18)) - lfid) / 2
  
  #write to df
  df[nrow(df) + 1,] = c(i, dist, lfid, tl, totadj, obsadj, nndistance)
}

summary(df) #the fids with NA have only 1 point; remove them later
df[is.na(df$dist),] #rows with NA have only 1 point,as I can't calculate a metric for them.

# Testing and plotting tsp outputs and points -----------------------------
#first plot the full set of points for a generic plot to serve as a backdrop
plot(nearnn$ptXutm[nearnn$fid == 1], nearnn$ptYutm[nearnn$fid == 1])

#now choose the FID of interest and plot on same graph 
i=1001
points(nearnn$ptXutm[nearnn$fid == 1 & nearnn$ctrl %in% nearnn$ctrl[nearnn$fid == i]], nearnn$ptYutm[nearnn$fid == 1 & nearnn$ctrl %in% nearnn$ctrl[nearnn$fid == i]],col="red", pch=19)

#put lines:
ss = merge(nearnn[nearnn$fid == 1 & nearnn$ctrl %in% nearnn$ctrl[nearnn$fid == i],],nearnn[nearnn$fid == 1,],by = "ctrl_box",all.x=TRUE)
dm = crossdist(nearnn$ptXutm[nearnn$fid == i],
               nearnn$ptYutm[nearnn$fid == i],
               nearnn$ptXutm[nearnn$fid == i],
               nearnn$ptYutm[nearnn$fid == i])
dmtsp = as.TSP(dm)
dmtsprout = solve_TSP(dmtsp, method = "farthest_insertion")
tl = tour_length(dmtsprout)
ord = as.numeric(attributes(TOUR(dmtsprout))$names)
lines(ss$ptXutm.y[order(ord)], ss$ptYutm.y[order(ord)], pch=16)
df[df$cntpts == 10,] # to only look at the list that has 10 points

# calculate additional indices for just the forest points portion ---------
summary(df)

df = df[df$tourlength != 0,] #get rid of plots with just 1 forested point -- some metrics require at least 2 forested points on a plot.

#calculate additional indices
df$obs_div_tot = df$observedadjacencies/df$totadjacencies #relative number of adjacencies
df$obs_pp_div_tot_pp = (df$observedadjacencies/df$cntpts)/(df$totadjacencies/52) # (#adj/#nts)/(max#adj/52);  there are 52 total points in each plot; I don't think this one is very meaningful, but I will keep it for future study

df$tourlength_pp = df$tourlength/df$cntpts #tour length per point
df$avedist_pp = df$dist/df$cntpts #average p-p distance/numpts

# Now calculate the average proportion of forest around each point on the full set of plots--------

forjoin = data.frame(in_sp@data) #just to simplify the below, get a small df with coordinates; 

df1$ones = 1 #this is the thing that indicates a point is forest
#create a join where I put the points that are forest onto the full dataset, and then add a "0" where there is no forest point. In other words, each point in the full dataset will be labeled with a 1 or a 0.
t0=merge(forjoin, df1, by.y=c("fid","ctrl"),by.x=c("FID_","ctrl_boxb"), all.x=TRUE)
t0$ones[is.na(t0$ones)]=0 # now each of the plots has 52 points, with a 0 (points with no forest) or a 1 (points with forest)
length(t0[,1]) #109148; 109148/52 =2099 #just to check.

#now loop through each plot, creating a marked point pattern for each FID, with all 52 points for each plot
#make container df
dfpp=data.frame(fid = integer(),aveprop = numeric(), sdprop = numeric(),stringsAsFactors = FALSE)
for(i in unique(t0$FID_)){
  #first make the points a ppp object
  xvec = t0[t0$FID_ == i,]$ptXutm
  yvec = t0[t0$FID_ == i,]$ptYutm
  pp=ppp(xvec, yvec, window=owin(c(min(xvec),max(xvec)),c(min(yvec),max(yvec))),marks = t0$ones[t0$FID_ == i])
  
  #now, calculate, per point, the average and sd canopy cover in an 18 m window around it. this includes the point itself, i.e., there are 9 values (maximum, if the point is internal to the circle) that go into the calculation. 
  aveprop = mean(applynbd(X=pp, R=18,function(Y,...) {mean(marks(Y))}))
  sdprop = sd(applynbd(X=pp, R=18,function(Y,...) {mean(marks(Y))}))
  dfpp[nrow(dfpp) + 1,] = c(i,aveprop, sdprop) #write to the output df
}

#now to calculate Prop Forest (PF) for each plot
#first, fix the NAs that were in the original dataset. If the comments made by the intepreter in the Access table suggest it's an inaccessible plot, I should remove the whole plot. Otherwise, assume a plot with no data is simply 100% nonforest.

#now I'm calculating the count
if(df1red_is_on == 0){
propfor_allFID = setDT(t0)[, lapply(.SD, sum), by = .(FID_),.SDcols=c(16)]
} else {
  propfor_allFID = setDT(t0)[, lapply(.SD, sum), by = .(FID_),.SDcols=c(18)]
  
}
propfor_allFID$propFor = propfor_allFID$ones/52 #there are 52 points; this attribute is the proportion forest (PF) on each plot.
  
# Now calculate the PAI ---------------------------------------------------
#note, for this one, I need to be aware that the NA's are ones where there was 0 forested points. It calculates when there's at least 1 forested point. if not, dfPAI$PAI will have NA. I need to remove these.
dfPAI=data.frame(fid = integer(),cntadj1_0 = integer(),cntadj1_1 = integer(), PAI=numeric(),stringsAsFactors = FALSE)
for(i in unique(t0$FID_)){
  #first make the points a ppp object
  xvec = t0[t0$FID_ == i,]$ptXutm
  yvec = t0[t0$FID_ == i,]$ptYutm
  pp=ppp(xvec, yvec, window=owin(c(min(xvec),max(xvec)),c(min(yvec),max(yvec))),marks = t0$ones[t0$FID_ == i])
  
  cd = crossdist(pp[pp$marks == 1],pp[pp$marks == 0])
  if(min(c(dim(cd)[1],dim(cd)[2])) == 0){cntadj1_0 = 0} else{
    
    cntadj1_0 = (sum(cd < 18)) #this is the count of the adjacencies of 1 with the zeros
  }
  lfid = sum(pp$marks == 1) #number of forest points in fid
  cntadj1_1 = (sum(pairdist(pp[pp$marks == 1]) < 18) - lfid) / 2 #this is the count of the adjacencies of 1 with 1
  dfPAI[nrow(dfPAI) + 1,] = c(i,cntadj1_0,cntadj1_1,cntadj1_1/(cntadj1_0+cntadj1_1)) #the PAI
}
length(dfPAI[,1]) #2099, so for PAI, I include all plots when I calculate my means but I have a pai of NA when the plot is 100% nf (denom is 0)
summary(dfPAI)
head(dfPAI[is.na(dfPAI$PAI),])

# Calculate the sampling statistics from the above outputs and results tables ----------------
###calculate mean, variance and sd, and se


funs <- function(x) {c(mean=mean(x), var=var(x), sd=sd(x), n = dim(df)[1],se=sd(x)/sqrt(dim(df)[1]))} #this calculates mean, variance, sd, n and se for each plot

df1red_is_on #Remember to pay attention to variable's status. See above where it is initialized. 

if(df1red_is_on == 0){
  dffull = df
  (resultstab = sapply(dffull, funs)) #this is the results table.
  write.csv(resultstab,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstab.csv")
  propfor_allFIDfull = propfor_allFID
  (resultstabPropfor = sapply(propfor_allFID, funs))#this is the results for the forest proportion per plot -- n=all fids
  write.csv(resultstabPropfor,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstabPropfor.csv")
  dfppfull = dfpp
  (resultstabPP = sapply(dfppfull, funs))
  write.csv(resultstabPP,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstabpp.csv")
  dfPAIfull = dfPAI
  dfPAIfull = na.omit(dfPAIfull)
  (resultstabPAI = sapply(dfPAIfull, funs))
  write.csv(resultstabPAI,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstabPAI.csv")
 
  print("full set")} else{
    #save the dfs so I can calculated covariances.
    dfred = df
  (resultstabReduced = sapply(dfred, funs)) #this is the results table.
    write.csv(resultstabReduced,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstabReduced.csv")
    propfor_allFIDred = propfor_allFID
  (resultstabPropforReduced = sapply(propfor_allFIDred, funs))#this is the results fro the forest proportion per plot -- n=all fids
    write.csv(resultstabPropforReduced,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstabPropforReduced.csv")
    dfppred = dfpp
  (resultstabPPReduced = sapply(dfppred, funs))
    write.csv(resultstabPPReduced,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstabPPReduced.csv")
    dfPAIred = dfPAI
    dfPAIred = na.omit(dfPAIred)
    (resultstabPAIReduced = sapply(dfPAIred, funs))
    write.csv(resultstabPAIReduced,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstabPAIReduced.csv")
    
    print("reduced set")}

} #end of loop that calculates results tables for the full dataset, and the reduced dataset. 

# now to calculate estimates of per-plot change in metric values, and to export results tables with pertinent information  --------------------


#note that all differences are calculated as reduced - full
dfPAImerge = merge(dfPAIred,dfPAIfull, by = "fid")
dfPAImerge = cbind(dfPAImerge, dfPAImerge[,4]-dfPAImerge[,7])
(resultstabPAImerge = sapply(dfPAImerge[,c(1,4,7,8)], funs))
write.csv(resultstabPAImerge,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstabPAImerged.csv")

dfppmerge = merge(dfppred,dfppfull, by = "fid")
dfppmerge = cbind(dfppmerge, dfppmerge[,c(2,3)]-dfppmerge[,c(4,5)])
names(dfppmerge) = c("fid","aveprop.x","sdprop.x","aveprop.y", "sdprop.y", "aveprop.xdif", "sdprop.xdif")
(resultstabppmerge = sapply(dfppmerge[,c(1,1:7)], funs))
write.csv(resultstabppmerge,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstabppmerged.csv")

propfor_allFIDfullmerge = merge(propfor_allFIDred,propfor_allFIDfull, by = "FID_")
propfor_allFIDfullmerge = cbind(propfor_allFIDfullmerge, propfor_allFIDfullmerge[,c(3)]-propfor_allFIDfullmerge[,c(5)])
names(propfor_allFIDfullmerge) = c("FID_","ones.x","propFor.x", "ones.y"   , "propFor.y","propFor.xdif")
(resultstabpropfor_allFIDfullmerge = sapply(propfor_allFIDfullmerge[,c(1:6)], funs))
write.csv(resultstabpropfor_allFIDfullmerge,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstabpropfor_allFIDfullmerge.csv")

dfmerge = merge(dfred,dffull, by = "fid")
dfmerge = cbind(dfmerge, dfmerge[,c(2:11)]-dfmerge[,c(12:21)])
names(dfmerge)[22:31] = c("dist.xdif" ,"cntpts.xdif","tourlength.xdif"  ,"totadjacencies.xdif","observedadjacencies.xdif","nndistance.xdif" ,"obs_div_tot.xdif","obs_pp_div_tot_pp.xdif", "tourlength_pp.xdif","avedist_pp.xdif")
(resultstabdfmerge = sapply(dfmerge[,c(1,2:31)], funs))
write.csv(resultstabdfmerge,"T:\\FS\\RD\\NRS\\Science\\FIA\\Project\\Inventory\\GIS\\alisterRestricted\\landchange\\2019_Forests_special_issue\\for_figs_tables\\resultstabdfmerge.csv")



